#include <stdlib.h>
#include <netdb.h>

#include "odbiorca.h"

bool_t xdr_call_info(register XDR*, call_info*);

/*
 * funkcja obslugi przychodzacych rzadan wykonania procedury
 */

void callback(struct svc_req* rqstp, SVCXPRT* transp) {
	wiad_a wiadomosc_a;
	wiad_b wiadomosc_b;
	wiad_c wiadomosc_c;

	switch (rqstp->rq_proc) { 
		case NULLPROC: 
			if (svc_sendreply(transp, xdr_void, 0) == FALSE)
				    fprintf(stderr, "error: svc_sendreply() for NULLPROC failed\n");
			break;
	      	case callback_a:
			if (svc_getargs(transp, (xdrproc_t) xdr_wiad_a, (char*)&wiadomosc_a) == TRUE) {
				if (wiadomosc_a.liczba1==-1 && wiadomosc_a.liczba2==-1)
					exit(EXIT_SUCCESS);
					
				printf("Odebrano wiadomosc A:\n %d\n %d\n", wiadomosc_a.liczba1, wiadomosc_a.liczba2);
				
				svc_freeargs(transp, (xdrproc_t) xdr_wiad_a, (caddr_t) &wiadomosc_a);					
			}
			break;
	      	case callback_b:
			if (svc_getargs(transp, xdr_wiad_b, (char*)&wiadomosc_b) == TRUE) {
				printf("Odebrano wiadomosc B:\n %c\n %c\n", wiadomosc_b.znak1, wiadomosc_b.znak2);
				svc_freeargs(transp, (xdrproc_t) xdr_wiad_b, (caddr_t) &wiadomosc_b);					
			}
			break;
	      	case callback_c:
			if (svc_getargs(transp, xdr_wiad_c, (char*)&wiadomosc_c) == TRUE) {
				printf("Odebrano wiadomosc C:\n %d\n %d\n %c\n %c\n", wiadomosc_c.wiada.liczba1, wiadomosc_c.wiada.liczba2, wiadomosc_c.wiadb.znak1, wiadomosc_c.wiadb.znak2);
				svc_freeargs(transp, (xdrproc_t) xdr_wiad_c, (caddr_t) &wiadomosc_c);	
			}
	}
}

/*
 * funkcja glowna
 */

int main (int argc, char *argv[]) {
	char host[256];
	SVCXPRT *transp; 
	call_info callback_info;
	int stat;
	struct hostent* h;

	if (argc < 3) {
		fprintf(stderr, "usage: %s server_host msg_type [local_host_addr]\n", argv[0]);
		return EXIT_FAILURE;
	}
	
	if (argv[2][0]<'A' || argv[2][0]>'C') {
		fprintf(stderr, "unrecognized message type\n");
		return EXIT_FAILURE;
	}
	
	if (argc==4)
		strncpy(callback_info.hostname, argv[3], 256);
	else {
		gethostname(host, 256);
		h = gethostbyname(host);
		strncpy(callback_info.hostname, h->h_name, 256);
	}
	
	if ((transp = svcudp_create(RPC_ANYSOCK)) == NULL ) { 
		fprintf(stderr, "error: svcudp_create() failed\n"); 
		return EXIT_FAILURE;
	}

	callback_info.prognum = svc_register_transient(transp, EVENT_SERVICE_VERS, callback, IPPROTO_UDP);

	if (callback_info.prognum == 0) { 
	    	fprintf(stderr, "error: couldn't register transient servive\n"); 
		svc_destroy(transp); 
		return EXIT_FAILURE;
	}

	callback_info.versnum = EVENT_SERVICE_VERS;
	callback_info.procnum = argv[2][0] - 'A' + callback_a;
	stat = callrpc(argv[1], EVENT_SERVICE_PROG, EVENT_SERVICE_VERS, zarejestruj, xdr_call_info, &callback_info, xdr_void, (xdrproc_t)NULL); 
	if (stat != 0) {
	      	fprintf(stderr, "error callrpc(): %s\n", clnt_sperrno((enum clnt_stat)stat));
		svc_unregister(callback_info.prognum, EVENT_SERVICE_VERS);
		svc_destroy(transp);
		return EXIT_FAILURE;
	}
	svc_run();
}
