#include "serwer.h"
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

struct odbiorca {
	struct odbiorca* nast;
	call_info info;
};

struct odbiorca *odbiorcyA, *odbiorcyB, *odbiorcyC;

pthread_mutex_t mutexA, mutexB, mutexC;

/*
 * procedura wywolujaca metode na zdalnym kliencie okreslonym przez parametr call_infop
 */

void docallback(call_info* call_infop, xdrproc_t encode, char* msg) {
	CLIENT *cl;
	static const struct timeval timeout = {0, 0};
	
	cl = clnt_create(call_infop->hostname, call_infop->prognum, call_infop->versnum, "udp"); 
	
        if (cl == NULL) { 
	 	clnt_pcreateerror(call_infop->hostname); 
		return; 
	} 
	
	clnt_call(cl, call_infop->procnum, encode, msg, xdr_void, NULL, timeout);
		 
	clnt_destroy(cl);
}

/*
 * procedura obslugi sygnalu SIGINT
 */

void sigint() {
	struct odbiorca* iterator;
	struct odbiorca* odb[3];
	struct wiad_a argp;
	int i;

	odb[0] = odbiorcyA;
	odb[1] = odbiorcyB;
	odb[2] = odbiorcyC;
	
	argp.liczba1 = argp.liczba2 = -1;
	
	pthread_mutex_lock(&mutexA);
	pthread_mutex_lock(&mutexB);	
	pthread_mutex_lock(&mutexC);

	for (i=0; i<3; i++) {
		iterator = odb[i];
	
		while (iterator) {
			iterator->info.procnum = callback_a;
			docallback(&iterator->info, xdr_wiad_a, (char*)&argp);
			iterator = iterator->nast;
		}
	}
	
	exit(EXIT_SUCCESS);
}

/*
 * procedura sprawdzajaca aktywnosc klientow, obslugiwana przez osobny watek
 */

void check_thread() {
	struct odbiorca* iterator, *old;
	struct odbiorca* odb[3];
	int i;

	for (;;) {
		sleep(10);
		
		odb[0] = odbiorcyA;
		odb[1] = odbiorcyB;
		odb[2] = odbiorcyC;
		
		for (i=0; i<3; i++) {
			iterator = odb[i];
			old = 0;
		
			switch (i) {
				case 0:
					pthread_mutex_lock(&mutexA);
					break;
				case 1:
					pthread_mutex_lock(&mutexB);
					break;
				case 2:
					pthread_mutex_lock(&mutexC);
			}
		
			while (iterator) {
				if (!keepalive(&iterator->info)) {
					printf("Wykryto rozlaczonego odbiorce\n");
					fflush(stdout);
					if (old) {
						old->nast = iterator->nast;
						free(iterator);
						iterator = old->nast;
					} else {
						if (iterator==odbiorcyA)
							odbiorcyA = odbiorcyA->nast;
						else if (iterator==odbiorcyB)
							odbiorcyB = odbiorcyB->nast;
						else
							odbiorcyC = odbiorcyC->nast;
								
						old = iterator->nast;
						free(iterator);
						iterator = old;
						old = 0;
					}
					continue;
				}
				
				old = iterator;
				iterator = iterator->nast;
			}
			
			pthread_mutex_unlock(&mutexA);
			pthread_mutex_unlock(&mutexB);
			pthread_mutex_unlock(&mutexC);
		}
	}
}

/*
 * funkcja sprawdzajaca dostepnosc konkretnego klienta, wykorzystuje wywolanie procedury NULL
 */

int keepalive(call_info* call_infop) {
	CLIENT *cl;
	enum clnt_stat stat;
	static const struct timeval timeout = {10, 0};
	
	cl = clnt_create(call_infop->hostname, call_infop->prognum, call_infop->versnum, "udp"); 
	
        if (cl == NULL)
		return 0;
	
	stat = clnt_call(cl, NULLPROC, xdr_void, NULL, xdr_void, NULL, timeout);

	clnt_destroy(cl);
		 
	if (stat != RPC_SUCCESS) {
	    	return 0;
	}
	
	return 1;
}

/*
 * obsluga przychodzacych wiadomosci typu pierewszego - odebranie oraz wyslanie do zainteresowanych odbiorcow
 */

void* wyslij_a_1_svc(wiad_a *argp, struct svc_req *rqstp) {
	static char * result;
	struct odbiorca* iterator;

	printf("Odebrano wiadomosc A:\n %d\n %d\n", argp->liczba1, argp->liczba2);

	pthread_mutex_lock(&mutexA);

	iterator = odbiorcyA;

	while (iterator) {
		docallback(&iterator->info, xdr_wiad_a, (char*)argp);
		iterator = iterator->nast;
	}

	pthread_mutex_unlock(&mutexA);

	return((void *) &result);
}

/*
 * obsluga przychodzacych wiadomosci typu drugiego - odebranie oraz wyslanie do zainteresowanych odbiorcow
 */

void* wyslij_b_1_svc(wiad_b *argp, struct svc_req *rqstp) {
	static char * result;
	struct odbiorca* iterator;

	printf("Odebrano wiadomosc B:\n %c\n %c\n", argp->znak1, argp->znak2);
	
	pthread_mutex_lock(&mutexB);
	
	iterator = odbiorcyB;
	
	while (iterator) {
		docallback(&iterator->info, xdr_wiad_b, (char*)argp);
		iterator = iterator->nast;
	}

	pthread_mutex_unlock(&mutexB);

	return((void *) &result);
}

/*
 * obsluga przychodzacych wiadomosci typu trzeciego - odebranie oraz wyslanie do zainteresowanych odbiorcow
 */

void* wyslij_c_1_svc(wiad_c *argp, struct svc_req *rqstp) {
	static char * result;
	struct odbiorca* iterator;

	printf("Odebrano wiadomosc C:\n %d\n %d\n %c\n %c\n", argp->wiada.liczba1, argp->wiada.liczba2, argp->wiadb.znak1, argp->wiadb.znak2);

	pthread_mutex_lock(&mutexC);

	iterator = odbiorcyC;

	while (iterator) {
		docallback(&iterator->info, xdr_wiad_c, (char*)argp);
		iterator = iterator->nast;
	}
	
	pthread_mutex_unlock(&mutexC);
	
	return((void *) &result);
}

/*
 * obsluga zglaszajacych sie odbiorcow - instancja struktury opisujacej odbiorce zostaje dopisana do listy odbiorcow zapisanych na dany
 * typ widomosci
 */

void* zarejestruj_1_svc(call_info *argp, struct svc_req *rqstp) {
	static char* result;
	struct odbiorca **tmp, *nowy;

	printf("Przyjmuje nowe zgloszenie ");

	switch (argp->procnum) {
		case callback_a:
			pthread_mutex_lock(&mutexA);
			tmp = &odbiorcyA;
			printf("na typ A...\n");
			break;
		case callback_b:
			pthread_mutex_lock(&mutexB);
			tmp = &odbiorcyB;
			printf("na typ B...\n");
			break;
		case callback_c:
			pthread_mutex_lock(&mutexC);
			tmp = &odbiorcyC;
			printf("na typ C...\n");
			break;
		default:
			fprintf(stderr, "Fatal error! (procnum == %d)\n", argp->procnum);
			exit(EXIT_FAILURE);
	}

	nowy = malloc(sizeof(struct odbiorca));
	memcpy(&nowy->info, argp, sizeof(call_info));

	nowy->nast = *tmp;
	*tmp = nowy;

	pthread_mutex_unlock(&mutexA);
	pthread_mutex_unlock(&mutexB);
	pthread_mutex_unlock(&mutexC);

	return((void *) &result);
}
