#include "serwer.h"
#include <stdio.h>
#include <stdlib.h>
#include <rpc/pmap_clnt.h>
#include <string.h>
#include <memory.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <pthread.h>
#include <signal.h>

void check_thread();

void sigint();

extern pthread_mutex_t mutexA, mutexB, mutexC;

/*
 * wygenerowana przez rpcgen'a i zmodyfikowana funkcja obslugi przychodzacych rzadan wywolania procedury
 */

static void event_service_prog_1(struct svc_req *rqstp, register SVCXPRT *transp) {
	union {
		wiad_a wyslij_a_1_arg;
		wiad_b wyslij_b_1_arg;
		wiad_c wyslij_c_1_arg;
		call_info zarejestruj_1_arg;
	} argument;
	char *result;
	xdrproc_t _xdr_argument, _xdr_result;
	char *(*local)(char *, struct svc_req *);

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void) svc_sendreply (transp, (xdrproc_t) xdr_void, (char *)NULL);
		return;

	case wyslij_a:
		_xdr_argument = (xdrproc_t) xdr_wiad_a;
		_xdr_result = (xdrproc_t) xdr_void;
		local = (char *(*)(char *, struct svc_req *)) wyslij_a_1_svc;
		break;

	case wyslij_b:
		_xdr_argument = (xdrproc_t) xdr_wiad_b;
		_xdr_result = (xdrproc_t) xdr_void;
		local = (char *(*)(char *, struct svc_req *)) wyslij_b_1_svc;
		break;

	case wyslij_c:
		_xdr_argument = (xdrproc_t) xdr_wiad_c;
		_xdr_result = (xdrproc_t) xdr_void;
		local = (char *(*)(char *, struct svc_req *)) wyslij_c_1_svc;
		break;

	case zarejestruj:
		_xdr_argument = (xdrproc_t) xdr_call_info;
		_xdr_result = (xdrproc_t) xdr_void;
		local = (char *(*)(char *, struct svc_req *)) zarejestruj_1_svc;
		break;

	default:
		svcerr_noproc (transp);
		return;
	}
	
	bzero((char *)&argument, sizeof (argument));
	
	if (!svc_getargs (transp, (xdrproc_t) _xdr_argument, (caddr_t) &argument)) {
		svcerr_decode (transp);
		return;
	}
	result = (*local)((char *)&argument, rqstp);
	if (result != NULL && rqstp->rq_proc==zarejestruj && !svc_sendreply(transp, (xdrproc_t) _xdr_result, result))
		svcerr_systemerr (transp);

	if (!svc_freeargs (transp, (xdrproc_t) _xdr_argument, (caddr_t) &argument))
		fprintf (stderr, "%s", "unable to free arguments\n");
}

/*
 * funkcja glowna
 */

int main (int argc, char **argv) {
	register SVCXPRT *transp;
	pthread_t tid;

	pmap_unset (EVENT_SERVICE_PROG, EVENT_SERVICE_VERS);

	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		fprintf (stderr, "cannot create udp service\n");
		return EXIT_FAILURE;
	}
	if (!svc_register(transp, EVENT_SERVICE_PROG, EVENT_SERVICE_VERS, event_service_prog_1, IPPROTO_UDP)) {
		fprintf (stderr, "unable to register (EVENT_SERVICE_PROG, EVENT_SERVICE_VERS, udp)\n");
		return EXIT_FAILURE;
	}

	if (pthread_create(&tid, 0, check_thread, 0)) {
		fprintf(stderr, "unable to create thread\n");
		return EXIT_FAILURE;
	}

	signal(SIGINT, sigint);

	pthread_mutex_init(&mutexA, 0);
	pthread_mutex_init(&mutexB, 0);
	pthread_mutex_init(&mutexC, 0);

	svc_run ();
	fprintf (stderr, "svc_run returned\n");
	return EXIT_FAILURE;
}
