package sr.madej.receiver;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import sr.madej.generated.*;
import sr.madej.generated.LoggerPackage.*;

public class Receiver extends LoggerListenerPOA implements ItemListener {
	private Logger logger;
	private LoggerListener href;

	private JTextArea tarea = new JTextArea(20, 50);
	private JScrollPane scrollPane = new JScrollPane(tarea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	
	private int lines = 0;
	
	public static void main(String[] args) {
		ORB orb = ORB.init( args, null );

		try {
			// pobranie zdalnej referancji do Logger'a
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
			
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
			Logger logger = LoggerHelper.narrow(ncRef.resolve_str("madej"));
			
			
			
			// udostepnienie na zewnatrz LoggerListener'a
			POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
	
			Receiver r = new Receiver(logger);
			
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(r);
	
			LoggerListener href = LoggerListenerHelper.narrow(ref);
			r.href = href;
			
			
		
			// rejestracja
			logger.addLoggerListener(href, LogPriority.from_int(0));
		} catch (Exception ex) {
			System.err.println(ex);
			System.exit(1);
		}
	}
	
	Receiver(Logger log) {
		logger = log;
		
		String[] items = new String[3];
		
		items[0] = "DEBUG";
		items[1] = "WARNING";
		items[2] = "PANIC";
		
		JFrame frame = new JFrame();
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		
		tarea.setEditable(false);
		
		JComboBox menu = new JComboBox(items);
		
		menu.addItemListener(this);
		
		panel.add(scrollPane);
		panel.add(menu);
		frame.add(panel);
		frame.pack();
		frame.setVisible(true);
		
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				try {
					logger.removeLoggerListener(href);
				} catch (Exception ex) { }
				System.exit(0);
			}
		});
	}
	
	public void itemStateChanged(ItemEvent e) {
		try {
			if (e.getItem()=="DEBUG")
				logger.alterLoggerListener(href, LogPriority.from_int(0));
			else if (e.getItem()=="WARNING")
				logger.alterLoggerListener(href, LogPriority.from_int(1));
			else
				logger.alterLoggerListener(href, LogPriority.from_int(2));
		} catch (Exception ex) {
			System.err.println(ex);
			System.exit(1);
		}
	}
	
	public void processLogRecord(LogRecord lr) {
		lines++;
		if (lines>50)
			tarea.setText(tarea.getText().substring(tarea.getText().indexOf("\n")+1) + new Date(lr.time) + ": " + lr.message + "\n");
		else
			tarea.append(new Date(lr.time) + ": " + lr.message + "\n");
		scrollPane.getVerticalScrollBar().setValue(scrollPane.getVerticalScrollBar().getMaximum());
	}
}
