package sr.madej.server;

import org.omg.CORBA.ORB;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;

import sr.madej.generated.*;
import sr.madej.generated.LoggerPackage.*;

import java.util.*;
import java.util.concurrent.*;

public class Server extends LoggerPOA {
	private Hashtable<LoggerListener, LogPriority> listeners = new Hashtable<LoggerListener, LogPriority>();
	private Executor exec = Executors.newCachedThreadPool();

	public static void main(String[] args) {
		ORB orb = ORB.init( args, null );

		try {
			POA rootpoa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
	
			Server functionsImpl = new Server();
	
			org.omg.CORBA.Object ref = rootpoa.servant_to_reference(functionsImpl);
	
			Logger href = LoggerHelper.narrow(ref);
	
			org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
	
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);
			NameComponent path[] = ncRef.to_name("madej");
			ncRef.rebind(path, href);
		} catch (Exception ex) {
			System.err.println(ex);
		}
			
		orb.run();
	}
	
	public synchronized void log(LogRecordHolder rec) {
		Enumeration<LoggerListener> elements = listeners.keys();
                while (elements.hasMoreElements()) {
                	LoggerListener ll = elements.nextElement();
                	LogPriority lp = listeners.get(ll);
                	
                	if (lp.value()==rec.value.priority.value())
                		exec.execute(new LogThread(ll, rec.value, this));
                }
	}
  	
  	public synchronized void addLoggerListener(LoggerListener ll, LogPriority priority) throws AlreadyAdded {
  		if (listeners.containsKey(ll))
  			throw new AlreadyAdded();
  		else
  			listeners.put(ll, priority);
  	}
  	
  	public synchronized void alterLoggerListener(LoggerListener ll, LogPriority priority) throws NotPresent {
  		if (!listeners.containsKey(ll))
  			throw new NotPresent();
  		else
  			listeners.put(ll, priority);
  	}
  	
  	public synchronized void removeLoggerListener(LoggerListener ll) throws NotPresent {
  		if (listeners.remove(ll)==null)
  			throw new NotPresent();
  	}
}

class LogThread implements Runnable {
	private LoggerListener ll;
	private LogRecord lr;
	private Server s;

	LogThread(LoggerListener ll, LogRecord lr, Server s) {
		this.ll = ll;
		this.lr = lr;
		this.s = s;
	}

	public void run() {
		try {
			ll.processLogRecord(lr);
		} catch(Exception ex) {
			try {
				s.removeLoggerListener(ll);
			} catch(Exception ex2) { }
		}
	}
}
