#include <iostream.h>
#include <signal.h>
#include <hash_map.h>
#include <list.h>
#include <string>
#include <omnithread.h>

#include "generated/noteboard.hh"

struct eqstr {
	bool operator()(char* s1, char* s2) const {
    		return strcmp(s1, s2) == 0;
  	}
};

static CORBA::Boolean bindObjectToName(CORBA::ORB_ptr, CORBA::Object_ptr);

using namespace org::dsrg::labs::corba::noteboard::generated;

CosNaming::Name objectName;
CosNaming::NamingContext_var rootContext;

class NoteBoardImpl : public POA_org::dsrg::labs::corba::noteboard::generated::NoteBoard {
public:
	void appendText(const char*, const char*);
	void unRegister(NoteBoardListener_ptr, const User&);
	void _cxx_register(NoteBoardListener_ptr, const User&);
private:
	hash_map<char*, NoteBoardListener_ptr, hash<char*>, eqstr> users;
	omni_mutex reg;
	omni_mutex unreg;
	omni_mutex app;
	omni_mutex map;
};

void NoteBoardImpl::appendText(const char* text, const char* nick) {
	omni_mutex_lock l(app);
	cout << "New text appended\n";
	char* tmp = strdup(nick);
	list<char*> rem;
	
	map.lock();
	hash_map<char*, NoteBoardListener_ptr, hash<char*>, eqstr>::iterator it;
	it = users.find(tmp);
	free(tmp);
	
	if (it == users.end()) {
		map.unlock();
		cerr << "  unknown client, throwing exception\n";
		throw org::dsrg::labs::corba::noteboard::generated::NoteBoard::UserRejectedException();
	} else 
		for (it = users.begin(); it != users.end(); it++) {
			try {
				it->second->onNewText(text);
			} catch (...) {
				cerr << "  client didn't respond, removing from list\n";
				rem.push_front(it->first);
			}
		}
	
	while (rem.size() > 0) {
		hash_map<char*, NoteBoardListener_ptr, hash<char*>, eqstr>::iterator it2 = users.find(rem.front());
	
		users.erase(it2->first);
		NoteBoardListener_Helper::release(it2->second);
		free(it2->first);
		
		rem.pop_front();
	}
	map.unlock();
}

void NoteBoardImpl::unRegister(NoteBoardListener_ptr listener, const User& usr) {
	omni_mutex_lock l(unreg);
	cout << "Client unregistered\n";
	char* tmp = strdup((const char*)usr.nick);
	
	map.lock();
	hash_map<char*, NoteBoardListener_ptr, hash<char*>, eqstr>::iterator it;
	it = users.find(tmp);	
	
	if (it != users.end()) {
		NoteBoardListener_ptr ptr = it->second;
		NoteBoardListener_Helper::release(ptr);
		users.erase(tmp);
		map.unlock();
		free(it->first);
		free(tmp);
	} else {
		map.unlock();
		free(tmp);
		cerr << "  unknown client, throwing exception\n";
		throw org::dsrg::labs::corba::noteboard::generated::NoteBoard::UserRejectedException();
	}
}

void NoteBoardImpl::_cxx_register(NoteBoardListener_ptr listener, const User& usr) {
	omni_mutex_lock l(reg);
	cout << "New client registered\n";
	char* tmp = strdup((const char*)usr.nick);
	
	map.lock();
	if (users.find(tmp) == users.end()) {
		users[tmp] = NoteBoardListener::_duplicate(listener);
		map.unlock();
	} else {
		map.unlock();
		free(tmp);
		cerr << "  client already registered, throwing exception\n";
		throw org::dsrg::labs::corba::noteboard::generated::NoteBoard::UserRejectedException();
	}
}

extern "C" {
	void sighandler(int);
}

void sighandler(int) {
	rootContext->unbind(objectName);
}

int main(int argc, char **argv) {
	signal(SIGINT, sighandler);

	try {
    		CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

    		CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    		PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

    		NoteBoardImpl* nt = new NoteBoardImpl();

    		PortableServer::ObjectId_var ntid = poa->activate_object(nt);

    		// Obtain a reference to the object, and register it in
    		// the naming service.
    		obj = nt->_this();
    		

    		CORBA::String_var x;
    		x = orb->object_to_string(obj);
    		cout << x << "\n";

    		if (!bindObjectToName(orb, obj)) {
    			cerr << "Couldn't bind object in naming service\n";
      			return 1;
      		}

    		nt->_remove_ref();

    		PortableServer::POAManager_var pman = poa->the_POAManager();
    		pman->activate();

    		orb->run();
  	} catch(CORBA::SystemException&) {
    		cerr << "Caught CORBA::SystemException." << endl;
  	} catch (CORBA::Exception&) {
    		cerr << "Caught CORBA::Exception." << endl;
  	} catch (omniORB::fatalException& fe) {
    		cerr << "Caught omniORB::fatalException:" << endl;
    		cerr << "  file: " << fe.file() << endl;
    		cerr << "  line: " << fe.line() << endl;
    		cerr << "  mesg: " << fe.errmsg() << endl;
  	} catch (...) {
    		cerr << "Caught unknown exception." << endl;
  	}

  	return 0;
}

static CORBA::Boolean bindObjectToName(CORBA::ORB_ptr orb, CORBA::Object_ptr objref) {
  	try {
		// Obtain a reference to the root context of the Name service:
		CORBA::Object_var obj;
		obj = orb->resolve_initial_references("NameService");

		// Narrow the reference returned.
		rootContext = CosNaming::NamingContext::_narrow(obj);
		if (CORBA::is_nil(rootContext)) {
			cerr << "Failed to narrow the root naming context." << endl;
			return 0;
		}
		
		objectName.length(1);
		objectName[0].id   = (const char*) "madej";   // string copied
		objectName[0].kind = (const char*) ""; // string copied
		
		try {
			rootContext->bind(objectName, objref);
		} catch (CosNaming::NamingContext::AlreadyBound& ex) {
			rootContext->rebind(objectName, objref);
		}
	} catch (CORBA::ORB::InvalidName& ex) {
    		// This should not happen!
    		cerr << "Service required is invalid [does not exist]." << endl;
    		return 0;
  	}

  	return 1;
}
