package org.dsrg.labs.corba.noteboard;

import org.dsrg.labs.corba.noteboard.generated.*;
import org.dsrg.labs.corba.noteboard.generated.NoteBoardPackage.*;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.omg.CORBA.ORB;
import org.omg.CORBA.NVList;
import org.omg.CORBA.Any;
import org.omg.CORBA.Request;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;

/**
 * Klient.
 */
public class NoteBoardClient extends NoteBoardListenerPOA {
	JFrame frame = new JFrame("NoteClient");
	JTextArea textArea = new JTextArea(30, 60);
	JButton bClear = new JButton("clear");
	JButton bAppend = new JButton("append");
	JButton bLogin = new JButton("login");
	JButton bLogout = new JButton("logout");

	JTextField userName = new JTextField(10);
	JTextField userNick = new JTextField(10);
	JTextField userLastName = new JTextField(10);
	JTextField textField = new JTextField(50);

	JScrollPane scrollPane;
	
	User user;
	org.omg.CORBA.Object noteRef;
	
	ORB myORB;
	org.omg.CORBA.Object objRef;
	
	int lines = 0;
	
	/**
	 * Konstruktor.
	 */
	NoteBoardClient(String[] args) {
		bClear.addActionListener(new ActionHandler());
		bLogin.addActionListener(new ActionHandler());
		bLogout.addActionListener(new ActionHandler());
		bAppend.addActionListener(new ActionHandler());

		frame.addWindowListener(new ActionHandler());

		scrollPane = new JScrollPane(textArea, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		textArea.setEditable(false);
		frame.getContentPane().add(scrollPane, BorderLayout.CENTER);

		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));

		JPanel userPanel = new JPanel();
		userPanel.add(new JLabel("nick"));
		userPanel.add(userNick);
		userPanel.add(new JLabel("name"));
		userPanel.add(userName);
		userPanel.add(new JLabel("lastname"));
		userPanel.add(userLastName);
		userPanel.add(bLogin);
		userPanel.add(bLogout);
		bLogout.setEnabled(false);

		JPanel textPanel = new JPanel();
		textPanel.add(new JLabel("text"));
		textPanel.add(textField);
		textPanel.add(bAppend);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(bClear);

		bottomPanel.add(userPanel);
		bottomPanel.add(textPanel);
		bottomPanel.add(buttonPanel);

		frame.getContentPane().add(bottomPanel, BorderLayout.SOUTH);
		frame.pack();
		frame.setVisible(true);

		myORB = ORB.init(args, null);
    		try {
      			// Pobranie referencji do NoteBoard
      			org.omg.CORBA.Object ncRef = myORB.resolve_initial_references("NameService");

      			NamingContextExt nc = NamingContextExtHelper.narrow(ncRef);
			NameComponent path[] = nc.to_name("madej");
			objRef = nc.resolve(path);
			
			// Wyeksportowanie wlasnego obiektu
			POA rootpoa = POAHelper.narrow(myORB.resolve_initial_references("RootPOA"));
			rootpoa.the_POAManager().activate();
			
			noteRef = rootpoa.servant_to_reference(this);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * Klasa zagniezdzona dostarczajaca interfejsu dla rejestracji obslugi zdarzen GUI.
	 */
	class ActionHandler extends WindowAdapter implements ActionListener {
	
		/**
		 * Metoda wywolywana w trakcie wystapienia zdarzenia ActionEvent
		 *
		 * @param e obiekt opisujacy zdarzenie
		 */
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == bClear) {
				textArea.setText("");
				lines = 0;
			}

			if (e.getSource() == bAppend) {
				if (user == null) {
					log("user is null!!!");
					return;
				}
				if (objRef == null) {
					log("not connected to board!!!");
					return;
				}
				
				//call the appendText on the remote board
				
				Thread th = new Thread() {
					private String text = textField.getText();
					private String nick = user.nick;
				
					public void run () {
						NVList argList = myORB.create_list(2);
				
						Any arg1 = myORB.create_any();
						arg1.insert_string(text);
						argList.add_value("text", arg1, org.omg.CORBA.ARG_IN.value);
						
						Any arg2 = myORB.create_any();
						arg2.insert_string(nick);
						argList.add_value("nick", arg2, org.omg.CORBA.ARG_IN.value);
						
						Request thisReq = objRef._create_request(null, "appendText", argList, null);
						thisReq.exceptions().add(UserRejectedExceptionHelper.type());
						thisReq.invoke();
						if (thisReq.env().exception()!=null)
							displayError("User not registered!");
					}
				};
				
				th.start();
			}
			
			if (e.getSource() == bLogin) {
				if (objRef == null) {
					log("not connected to board!!!");
					return;
				}
		
			  	//create User instance
				
			  	user = new User(userLastName.getText(), userName.getText(), userNick.getText());

			  	//register in board
			  	//by calling register() on board interface
			  	
				NVList argList = myORB.create_list(2);
				
				Any arg1 = myORB.create_any();
				NoteBoardListenerHelper.insert(arg1, NoteBoardListenerHelper.narrow(noteRef));
				argList.add_value("listener", arg1, org.omg.CORBA.ARG_IN.value);
				
				Any arg2 = myORB.create_any();
				UserHelper.insert(arg2, user);
				argList.add_value("usr", arg2, org.omg.CORBA.ARG_IN.value);
				
				Request thisReq = objRef._create_request(null, "register", argList, null);
				thisReq.exceptions().add(UserRejectedExceptionHelper.type());
				thisReq.invoke();
				if (thisReq.env().exception()!=null) {
					displayError("User already registered!");
					user = null;
					return;
				}
			  	
			   	bLogin.setEnabled(false);
			   	bLogout.setEnabled(true);
			}

			if (e.getSource() == bLogout) {
			  	//unregister from board
			  	//by calling unregister() on board interface
			  	
				NVList argList = myORB.create_list(2);
				
				Any arg1 = myORB.create_any();
				arg1.insert_Object(noteRef);
				argList.add_value("listener", arg1, org.omg.CORBA.ARG_IN.value);
				
				Any arg2 = myORB.create_any();
				UserHelper.insert(arg2, user);
				argList.add_value("usr", arg2, org.omg.CORBA.ARG_IN.value);
				
				Request thisReq = objRef._create_request(null, "unRegister", argList, null);

				thisReq.exceptions().add(UserRejectedExceptionHelper.type());
				thisReq.invoke();
				if (thisReq.env().exception()!=null)
					displayError("User not registered!");
				
			  	user = null;
			  	
			  	bLogin.setEnabled(true);
			   	bLogout.setEnabled(false);
			}
		}

		/**
		 * Metoda wywolywana w trakcie wystapienia zdarzenia WindowEvent
		 *
		 * @param e obiekt opisujacy zdarzenie
		 */
		public void windowClosing(WindowEvent e) {
			//logout - may throw exception
			//and make me not accessible remotely
			//disconnect remote object
			
			try {
				NVList argList = myORB.create_list(2);
					
				Any arg1 = myORB.create_any();
				arg1.insert_Object(noteRef);
				argList.add_value("listener", arg1, org.omg.CORBA.ARG_IN.value);
				
				Any arg2 = myORB.create_any();
				UserHelper.insert(arg2, user);
				argList.add_value("usr", arg2, org.omg.CORBA.ARG_IN.value);
				
				Request thisReq = objRef._create_request(null, "unRegister", argList, null);
				thisReq.invoke();
			} catch (Exception ex) { }
			
			objRef = null;
			System.exit(0);
		}
	}

	/**
	 * Metoda wywolywana w wyniku dopisania nowej linii tekstu na tablicy.
	 *
	 * @param text tekst
	 */
	public synchronized void onNewText(String text) {
                lines++;
                
                if (lines>50) {
                	lines = 50;
                        textArea.setText(textArea.getText().substring(textArea.getText().indexOf("\n")+1) + text+"\n");
                } else
                        textArea.append(text+"\n");
                scrollPane.getVerticalScrollBar().setValue(scrollPane.getVerticalScrollBar().getMaximum());
	}

	/**
	 * Metoda logujaca.
	 * 
	 * @param text tekst do zalogowania
	 */
	private void log(String text) {
		textArea.append("local:" + text + "\n");
	}

	/**
	 * Statyczna metoda pozwalajaca na uruchomienie programu.
	 *
	 * @param args tablica argumentow
	 */
	public static void main(String[] args) {
		try {
			new NoteBoardClient(args);
			return;
		} catch (Exception e) { }
	}

	/**
	 * Wyswietla okno dialogowe z zadanym tekstem.
	 *
	 * @param text tekst
	 */
	private synchronized void displayError(String text) {
		JOptionPane.showInternalMessageDialog(
			frame.getContentPane(),
			text,
			"error",
			JOptionPane.INFORMATION_MESSAGE);
	}
}
